unit UMath;

interface

type

  TVector = record
    x:Double;
    y:Double;
    z:Double;
  end;

  TMatrix = record
    m:array[0..3,0..3]of Double;
  end;

  function MatUnInit():TMatrix;
  function MatMul(a,b:TMatrix):TMatrix;
  function MatApply(a:TMatrix; b:TVector):TVector;
  function MatRotX( rad:Double ):TMatrix;
  function MatRotY( rad:Double ):TMatrix;
  function MatRotZ( rad:Double ):TMatrix;

implementation

function MatUnInit():TMatrix;
var
  m:TMatrix;
begin
  m.m[0,0]:=1; m.m[0,1]:=0; m.m[0,2]:=0; m.m[0,3]:=0;
  m.m[1,0]:=0; m.m[1,1]:=1; m.m[1,2]:=0; m.m[1,3]:=0;
  m.m[2,0]:=0; m.m[2,1]:=0; m.m[2,2]:=1; m.m[2,3]:=0;
  m.m[3,0]:=0; m.m[3,1]:=0; m.m[3,2]:=0; m.m[3,3]:=1;
  Result := m;
end;
function MatMul(a,b:TMatrix):TMatrix;
var
  m:TMatrix;
begin
  m.m[0,0] := a.m[0,0]*b.m[0,0] + a.m[0,1]*b.m[1,0] + a.m[0,2]*b.m[2,0] + a.m[0,3]*b.m[3,0];
  m.m[0,1] := a.m[0,0]*b.m[0,1] + a.m[0,1]*b.m[1,1] + a.m[0,2]*b.m[2,1] + a.m[0,3]*b.m[3,1];
  m.m[0,2] := a.m[0,0]*b.m[0,2] + a.m[0,1]*b.m[1,2] + a.m[0,2]*b.m[2,2] + a.m[0,3]*b.m[3,2];
  m.m[0,3] := a.m[0,0]*b.m[0,3] + a.m[0,1]*b.m[1,3] + a.m[0,2]*b.m[2,3] + a.m[0,3]*b.m[3,3];
  m.m[1,0] := a.m[1,0]*b.m[0,0] + a.m[1,1]*b.m[1,0] + a.m[1,2]*b.m[2,0] + a.m[1,3]*b.m[3,0];
  m.m[1,1] := a.m[1,0]*b.m[0,1] + a.m[1,1]*b.m[1,1] + a.m[1,2]*b.m[2,1] + a.m[1,3]*b.m[3,1];
  m.m[1,2] := a.m[1,0]*b.m[0,2] + a.m[1,1]*b.m[1,2] + a.m[1,2]*b.m[2,2] + a.m[1,3]*b.m[3,2];
  m.m[1,3] := a.m[1,0]*b.m[0,3] + a.m[1,1]*b.m[1,3] + a.m[1,2]*b.m[2,3] + a.m[1,3]*b.m[3,3];
  m.m[2,0] := a.m[2,0]*b.m[0,0] + a.m[2,1]*b.m[1,0] + a.m[2,2]*b.m[2,0] + a.m[2,3]*b.m[3,0];
  m.m[2,1] := a.m[2,0]*b.m[0,1] + a.m[2,1]*b.m[1,1] + a.m[2,2]*b.m[2,1] + a.m[2,3]*b.m[3,1];
  m.m[2,2] := a.m[2,0]*b.m[0,2] + a.m[2,1]*b.m[1,2] + a.m[2,2]*b.m[2,2] + a.m[2,3]*b.m[3,2];
  m.m[2,3] := a.m[2,0]*b.m[0,3] + a.m[2,1]*b.m[1,3] + a.m[2,2]*b.m[2,3] + a.m[2,3]*b.m[3,3];
  m.m[3,0] := a.m[3,0]*b.m[0,0] + a.m[3,1]*b.m[1,0] + a.m[3,2]*b.m[2,0] + a.m[3,3]*b.m[3,0];
  m.m[3,1] := a.m[3,0]*b.m[0,1] + a.m[3,1]*b.m[1,1] + a.m[3,2]*b.m[2,1] + a.m[3,3]*b.m[3,1];
  m.m[3,2] := a.m[3,0]*b.m[0,2] + a.m[3,1]*b.m[1,2] + a.m[3,2]*b.m[2,2] + a.m[3,3]*b.m[3,2];
  m.m[3,3] := a.m[3,0]*b.m[0,3] + a.m[3,1]*b.m[1,3] + a.m[3,2]*b.m[2,3] + a.m[3,3]*b.m[3,3];
  Result := m;
end;

function MatApply(a:TMatrix; b:TVector):TVector;
var
  v:TVector;
begin
  v.x := a.m[0,0]*b.x + a.m[0,1]*b.y + a.m[0,2]*b.z;
  v.y := a.m[1,0]*b.x + a.m[1,1]*b.y + a.m[1,2]*b.z;
  v.z := a.m[2,0]*b.x + a.m[2,1]*b.y + a.m[2,2]*b.z;
  Result := v;
end;

function MatRotX( rad:Double ):TMatrix;
var
  s,c:Double;
  m:TMatrix;
begin
  s := Sin(rad);
  c := Cos(rad);
  m.m[0,0]:=1; m.m[0,1]:= 0; m.m[0,2]:=0; m.m[0,3]:=0;
  m.m[1,0]:=0; m.m[1,1]:= c; m.m[1,2]:=s; m.m[1,3]:=0;
  m.m[2,0]:=0; m.m[2,1]:=-s; m.m[2,2]:=c; m.m[2,3]:=0;
  m.m[3,0]:=0; m.m[3,1]:= 0; m.m[3,2]:=0; m.m[3,3]:=1;
  Result:=m;
end;

function MatRotY( rad:Double ):TMatrix;
var
  s,c:Double;
  m:TMatrix;
begin
  s := Sin(rad);
  c := Cos(rad);
  m.m[0,0]:=c; m.m[0,1]:=0; m.m[0,2]:=-s; m.m[0,3]:=0;
  m.m[1,0]:=0; m.m[1,1]:=1; m.m[1,2]:= 0; m.m[1,3]:=0;
  m.m[2,0]:=s; m.m[2,1]:=0; m.m[2,2]:= c; m.m[2,3]:=0;
  m.m[3,0]:=0; m.m[3,1]:=0; m.m[3,2]:= 0; m.m[3,3]:=1;
  Result:=m;
end;

function MatRotZ( rad:Double ):TMatrix;
var
  s,c:Double;
  m:TMatrix;
begin
  s := Sin(rad);
  c := Cos(rad);
  m.m[0,0]:= c; m.m[0,1]:=s; m.m[0,2]:=0; m.m[0,3]:=0;
  m.m[1,0]:=-s; m.m[1,1]:=c; m.m[1,2]:=0; m.m[1,3]:=0;
  m.m[2,0]:= 0; m.m[2,1]:=0; m.m[2,2]:=1; m.m[2,3]:=0;
  m.m[3,0]:= 0; m.m[3,1]:=0; m.m[3,2]:=0; m.m[3,3]:=1;
  Result:=m;
end;


end.
